#
# Common settings and macros for build system. Some macros and ideas taken from:
# 
# http://www.cmake.org/Wiki/CMake_User_Contributed_Macros

# Our first operations we take from lisp, a programming language built
# entirely around lists. Two fundamental functions of lisp are car, which
# returns the first element of a list, and cdr, which returns a list minus
# the first element. Here we give implementations of car and cdr CMake macros.
# Although they are not as efficient as their counterparts in lisp, they can
# still be surprisingly useful.

MACRO(CAR var)
  SET(${var} ${ARGV1})
ENDMACRO(CAR)

# Get the rest of arguments from a list of arguments
MACRO(CDR var junk)
  SET(${var} ${ARGN})
ENDMACRO(CDR)

# Generates targets to soft link FILES from current directory  
# to INC_DIR

# Usage:

# SET(HDR_FILES head1.h head2.h head3.h)
# SOFTLINK(HeaderLinks ${HDR_FILES})

MACRO(SOFTLINK)
  CAR(TARGET ${ARGN})
  CDR(FILES ${ARGN})
  SET(OUTPUTS "")
  FOREACH(SRC ${FILES})
    GET_FILENAME_COMPONENT(SRCFILE ${SRC} NAME)
    SET(FROM ${CMAKE_CURRENT_SOURCE_DIR}/${SRCFILE})
    SET(TO ${INC_DIR}/${SRCFILE})
    ADD_CUSTOM_COMMAND(
      OUTPUT  ${TO}
      COMMAND /bin/ln
      ARGS    -sf ${FROM} ${TO}
      COMMENT "ln -sf ${FROM} ${TO}"
      )
    SET(OUTPUTS ${OUTPUTS} ${TO})
  ENDFOREACH(SRC ${FILES})
  ADD_CUSTOM_TARGET(${TARGET} DEPENDS ${OUTPUTS})
ENDMACRO(SOFTLINK)

# add binary directories to build

MACRO(ADD_BIN_DIRS)
  SET(PREFIX " + ")
  MESSAGE(STATUS "add_bin_dirs:")
  FOREACH(SUBDIR ${ARGN})
    MESSAGE(STATUS "${PREFIX}${SUBDIR}")
    ADD_SUBDIRECTORY(${SUBDIR} ${BIN_DIR})
  ENDFOREACH(SUBDIR ${ARGN})
ENDMACRO(ADD_BIN_DIRS)

# add library directories to build

MACRO(ADD_LIB_DIRS)
  SET(PREFIX " + ")
  MESSAGE(STATUS "add_lib_dirs:")
  FOREACH(SUBDIR ${ARGN})
    MESSAGE(STATUS "${PREFIX}${SUBDIR}")
    ADD_SUBDIRECTORY(${SUBDIR} ${LIB_DIR})
  ENDFOREACH(SUBDIR ${ARGN})
ENDMACRO(ADD_LIB_DIRS)

# add dependencies

MACRO(DEPEND)
  CAR(TARGET ${ARGN})
  CDR(DEPS ${ARGN})
  FOREACH(DEP ${DEPS})
    ADD_DEPENDENCIES(${TARGET} ${DEP})
  ENDFOREACH(DEP ${DEPS})
ENDMACRO(DEPEND)

# add header target to build

MACRO(ADD_HDR)
  SET(HDR_TARGETS ${HDR_TARGETS} ${ARGV0})
  SOFTLINK(${ARGN})
ENDMACRO(ADD_HDR)

# add binary target to build

MACRO(ADD_BIN)
  SET(BIN_TARGETS ${BIN_TARGETS} ${ARGV0})
  ADD_EXECUTABLE(${ARGN})
  DEPEND(${ARGV0} ${HDR_TARGETS})
ENDMACRO(ADD_BIN)

# add library target to build

MACRO(ADD_LIB)
  SET(LIB_TARGETS ${LIB_TARGETS} ${ARGV0})
  ADD_LIBRARY(${ARGN})
  DEPEND(${ARGV0} ${HDR_TARGETS})
ENDMACRO(ADD_LIB)

# collect all binary targets from given subdirectories

MACRO(GET_BIN_TARGETS)
  CAR(VAR ${ARGN})
  CDR(DIRS ${ARGN})
  SET(TARGETS "")
  FOREACH(SUBDIR ${DIRS})
    GET_DIRECTORY_PROPERTY(BIN_TARGET DIRECTORY ${SUBDIR} DEFINITION BIN_TARGETS)
    SET(TARGETS ${TARGETS} ${BIN_TARGET})
  ENDFOREACH(SUBDIR ${DIRS})
  SET(${VAR} ${TARGETS})
ENDMACRO(GET_BIN_TARGETS)

# collect all library targets from given subdirectories

MACRO(GET_LIB_TARGETS)
  CAR(VAR ${ARGN})
  CDR(DIRS ${ARGN})
  SET(TARGETS "")
  FOREACH(SUBDIR ${DIRS})
    GET_DIRECTORY_PROPERTY(LIB_TARGET DIRECTORY ${SUBDIR} DEFINITION LIB_TARGETS)
    SET(TARGETS ${TARGETS} ${LIB_TARGET})
  ENDFOREACH(SUBDIR ${DIRS})
  SET(${VAR} ${TARGETS})
ENDMACRO(GET_LIB_TARGETS)

# collect all header targets from given subdirectories

MACRO(GET_HDR_TARGETS)
  CAR(VAR ${ARGN})
  CDR(DIRS ${ARGN})
  SET(TARGETS "")
  FOREACH(SUBDIR ${DIRS})
    GET_DIRECTORY_PROPERTY(HDR_TARGET DIRECTORY ${SUBDIR} DEFINITION HDR_TARGETS)
    SET(TARGETS ${TARGETS} ${HDR_TARGET})
  ENDFOREACH(SUBDIR ${DIRS})
  SET(${VAR} ${TARGETS})
ENDMACRO(GET_HDR_TARGETS)

# create target to build group of targets

MACRO(GROUP_TARGET)
  CAR(TARGET ${ARGN})
  CDR(DEPS ${ARGN})
  ADD_CUSTOM_TARGET(${TARGET})
  FOREACH(DEP ${DEPS})
    ADD_DEPENDENCIES(${TARGET} ${DEP})
  ENDFOREACH(DEP ${DEPS})
ENDMACRO(GROUP_TARGET)
